@@ -1,7 +1,9 @@
 # This file allows all subdirectories in this directroy to loaded by Python
 # -*- coding: utf-8 -*-
 from .georasters import get_geo_info, map_pixel, aggregate, create_geotiff, align_rasters, \
-                        load_tiff, union, GeoRaster, RasterGeoError, RasterGeoTError, merge
+                        load_tiff, union, GeoRaster, RasterGeoError, RasterGeoTError, merge, \
+                        from_file
 
 __all__ = (['get_geo_info','map_pixel','aggregate','create_geotiff','align_rasters', \
-            'load_tiff', 'union', 'GeoRaster', 'RasterGeoError', 'RasterGeoTError', 'merge'])
+            'load_tiff', 'union', 'GeoRaster', 'RasterGeoError', 'RasterGeoTError', 'merge',
+            'from_file'])
