@@ -83,11 +83,7 @@ def aggregate(raster,NDV,block_size):
             costs = load_tiff(raster)
             costs2=aggregate(costs,NDV,(10,10))
     '''
-    raster2=np.where(raster==NDV,0,raster)
-    raster3=block_reduce(raster2,block_size,func=np.sum)
-    raster2=np.where(raster==NDV,NDV,0)
-    raster4=block_reduce(raster2,block_size,func=np.sum)
-    raster2=np.where(raster4<0,NDV,raster3)
+    raster2=block_reduce(raster2,block_size,func=np.ma.sum)
     return raster2
 
 # Function to write a new file.
@@ -182,9 +178,9 @@ def load_tiff(file):
     Usage:
             data=LoadTiffRaster(file)
     """
-    NDV, xsize, ysize, GeoT, Projection, DataType=GetGeoInfo(file)
+    NDV, xsize, ysize, GeoT, Projection, DataType=get_geo_info(file)
     data=gdalnumeric.LoadFile(file)
-    data=np.ma.masked_array(data, mask=data==NDV,fill_value=-np.inf)
+    data=np.ma.masked_array(data, mask=data==NDV,fill_value=NDV)
     return data
 
 class RasterGeoTError(Exception):
@@ -238,8 +234,8 @@ class GeoRaster():
     def __getslice__(self,i,j):
         return self.raster.__getslice__(i,j)
 
-    def __getattribute__(self, attr):
-        return eval('self.'+attr)
+#    def __getattribute__(self, attr):
+#        return eval('self.'+attr)
 
     def __lt__(self,other):
         if isinstance(other, GeoRaster):
@@ -281,7 +277,7 @@ class GeoRaster():
         return self
 
     def __neg__(self):
-        return GeoRaster(-self.raster, self.geot, nodata_value=self.nodata_value)
+        return GeoRaster(-self.raster, self.geot, nodata_value=self.nodata_value, projection = self.projection, datatype = self.datatype)
 
     def __add__(self, other):
         if isinstance(other,GeoRaster):
@@ -290,11 +286,11 @@ class GeoRaster():
                     ndv=self.nodata_value
                 else:
                     ndv=np.nan
-                return GeoRaster(self.raster+other.raster, self.geot, nodata_value=ndv)
+                return GeoRaster(self.raster+other.raster, self.geot, nodata_value=ndv, projection = self.projection, datatype = self.datatype)
             else:
                 raise RasterGeoTError("Rasters must have same geotransform. If needed first create union or allign them.")
         else:
-            return GeoRaster(self.raster+other, self.geot, nodata_value=self.nodata_value)
+            return GeoRaster(self.raster+other, self.geot, self.geot, nodata_value=self.nodata_value, projection = self.projection, datatype = self.datatype)
 
     def __radd__(self, other):
         return self.__add__(other)
@@ -312,11 +308,11 @@ class GeoRaster():
                     ndv=self.nodata_value
                 else:
                     ndv=np.nan
-                return GeoRaster(self.raster*other.raster, self.geot, nodata_value=ndv)
+                return GeoRaster(self.raster*other.raster, self.geot, nodata_value=ndv, projection = self.projection, datatype = self.datatype)
             else:
                 raise RasterGeoTError("Rasters must have same geotransform. If needed first create union or allign them.")
         else:
-            return GeoRaster(self.raster*other, self.geot, nodata_value=self.nodata_value)
+            return GeoRaster(self.raster*other, self.geot, self.geot, nodata_value=self.nodata_value, projection = self.projection, datatype = self.datatype)
 
     def __rmul__(self, other):
         return self.__mul__(other) 
@@ -328,17 +324,17 @@ class GeoRaster():
                     ndv=self.nodata_value
                 else:
                     ndv=np.nan
-                return GeoRaster(self.raster/other.raster, self.geot, nodata_value=ndv)
+                return GeoRaster(self.raster/other.raster, self.geot, nodata_value=ndv, projection = self.projection, datatype = self.datatype)
             else:
                 raise RasterGeoTError("Rasters must have same geotransform. If needed first create union or allign them.")
         else:
-            return GeoRaster(self.raster/other, self.geot, nodata_value=self.nodata_value)
+            return GeoRaster(self.raster/other, self.geot, nodata_value=self.nodata_value, projection = self.projection, datatype = self.datatype)
 
     def __rtruediv__(self, other):
         if isinstance(other,GeoRaster):
             return other.__truediv__(self)
         else:
-            return GeoRaster(other/self.raster, self.geot, nodata_value=self.nodata_value)
+            return GeoRaster(other/self.raster, self.geot, nodata_value=self.nodata_value, projection = self.projection, datatype = self.datatype)
 
     def __floordiv__(self, other):
         A=self/other
@@ -352,11 +348,11 @@ class GeoRaster():
                     ndv=self.nodata_value
                 else:
                     ndv=np.nan
-                return GeoRaster((other.raster/self.raster).astype(int), self.geot, nodata_value=ndv)
+                return GeoRaster((other.raster/self.raster).astype(int), self.geot, nodata_value=ndv, projection = self.projection, datatype = self.datatype)
             else:
                 raise RasterGeoTError("Rasters must have same geotransform. If needed first create union or allign them.")
         else:
-            return GeoRaster((other/self.raster).astype(int), self.geot, nodata_value=self.nodata_value)
+            return GeoRaster((other/self.raster).astype(int), self.geot, nodata_value=self.nodata_value, projection = self.projection, datatype = self.datatype)
 
     def __pow__(self,other):
         if isinstance(other,GeoRaster):
@@ -365,11 +361,11 @@ class GeoRaster():
                     ndv=self.nodata_value
                 else:
                     ndv=np.nan
-                return GeoRaster(self.raster**other.raster, self.geot, nodata_value=ndv)
+                return GeoRaster(self.raster**other.raster, self.geot, nodata_value=ndv, projection = self.projection, datatype = self.datatype)
             else:
                 raise RasterGeoTError("Rasters must have same geotransform. If needed first create union or allign them.")
         else:
-            return GeoRaster(self.raster**other, self.geot, nodata_value=self.nodata_value)
+            return GeoRaster(self.raster**other, self.geot, nodata_value=self.nodata_value, projection = self.projection, datatype = self.datatype)
 
     def to_tiff(self, filename):
         '''
@@ -578,7 +574,7 @@ class GeoRaster():
         Return computed Gini coefficient.
         """
         if self.count()>1:
-            xsort = sorted(self.raster.data[self.raster.data!=self.nodata_value].flatten()) # increasing order
+            xsort = sorted(self.raster.data[self.raster.mask==False].flatten()) # increasing order
             y = np.cumsum(xsort)
             B = sum(y) / (y[-1] * len(xsort))
             return 1 + 1./len(xsort) - 2*B
@@ -632,21 +628,39 @@ class GeoRaster():
         col2 = np.abs(radius/self.x_cell_size)
         row2 = np.abs(radius/self.y_cell_size)
         return GeoRaster(self.raster[max(row-row2, 0):min(row+row2+1, self.shape[0]), \
-                        max(col-col2, 0):min(col+col2+1, self.shape[1])], self.geot, nodata_value = self.nodata_value)
+                        max(col-col2, 0):min(col+col2+1, self.shape[1])], self.geot, nodata_value = self.nodata_value,\
+                        projection=self.projection, datatype = self.datatype)
 
     # Align GeoRasters
     def align(self,alignraster,how=np.mean,cxsize=None,cysize=None):
         '''
-        Align two rasters so that data overlaps by geographical location
-        Usage: (alignedraster_o, alignedraster_a) = AlignRasters(raster, alignraster, how=np.mean)
-        where 
-            raster: string with location of raster to be aligned
-            alignraster: string with location of raster to which raster will be aligned
-            how: function used to aggregate cells (if the rasters have different sizes)
-        It is assumed that both rasters have the same size
+        geo.align(geo2, how=np.mean)
+
+        Returns both georasters aligned and with the same pixelsize
         '''
         return align_georasters(self,alignraster,how=how,cxsize=cxsize,cysize=cysize)
 
+    def aggregate(self,block_size):
+        '''
+        geo.aggregate(block_size)
+
+        Returns copy of raster aggregated to smaller resolution, by adding cells.
+        '''
+        raster2=block_reduce(self.raster,block_size,func=np.ma.sum)
+        return GeoRaster(raster2, self.geot, nodata_value=self.nodata_value,\
+                        projection=self.projection, datatype = self.datatype)
+
+    def block_reduce(self,block_size, how=np.ma.mean):
+        '''
+        geo.block_reduce(block_size, how=func)
+
+        Returns copy of raster aggregated to smaller resolution, by adding cells.
+        Default: func=np.ma.mean
+        '''
+        raster2=block_reduce(self.raster,block_size,func=how)
+        return GeoRaster(raster2, self.geot, nodata_value=self.nodata_value,\
+                        projection=self.projection, datatype = self.datatype)
+
 # Union of rasters
 def union(rasters):
     """
@@ -771,3 +785,19 @@ def align_georasters(raster,alignraster,how=np.mean,cxsize=None,cysize=None):
     else:
         print("Rasters need to be in same projection")
         return (-1,-1)
+
+# Test if two GeoRasters are in same GeoT and have same projection
+def is_geovalid(grasterlist):
+    if np.sum(map(isinstance,grasterlist,[GeoRaster for i in range(len(grasterlist))]))==len(grasterlist):
+        graster0 = grasterlist[-1]
+        while grasterlist !=[]:
+            grasterlist = grasterlist[:-1]
+            graster1 = grasterlist[-1]
+            if graster1.geot == graster0.geot and graster1.projection.ExportToPrettyWkt() == graster0.ExportToPrettyWkt():
+                return 0
+            else:
+                raise RasterGeoTError("Rasters must have same geotransform and projection.")
+                return 1
+    else:
+        raise RasterGeoTError("List must contain only GeoRasters.")
+
